// -*- compile-command: "gcc -o simple_udev simple_udev.c -ludev" -*-
// Use subject to GPLv3.  Copyright 2011 George Magiros

/* Linux */
#include <linux/hidraw.h>

/* Unix */
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

/* C */
#include <stdio.h>
#include <stdlib.h>

/* udev */
#include <libudev.h>

// VID and PID of the USB device to locate
#define VID "16c0"
#define PID "05dc"

int main(int argc, char **argv)
{
  int fd, i, n, ret;
  unsigned char buf[1024];

  struct udev *udev;
  struct udev_enumerate *enumerate;
  struct udev_list_entry *devices, *dev_list_entry;
  struct udev_device *dev, *dev_int;
  
  udev = udev_new();
  if (!udev) return 1;
  enumerate = udev_enumerate_new(udev);
  udev_enumerate_add_match_subsystem(enumerate, "hidraw");
  udev_enumerate_scan_devices(enumerate);
  devices = udev_enumerate_get_list_entry(enumerate);

  fd = -1;
  udev_list_entry_foreach(dev_list_entry, devices) {
    const char *sys, *node, *vid, *pid, *vendor, *device, *serial;
    const char *intnum, *interface;

    sys = udev_list_entry_get_name(dev_list_entry);
    dev = udev_device_new_from_syspath(udev, sys);
    node = udev_device_get_devnode(dev);

    dev_int = udev_device_get_parent_with_subsystem_devtype(
                    dev, "usb", "usb_interface");
    if (!dev_int) return 1;
    intnum = udev_device_get_sysattr_value(dev_int,"bInterfaceNumber");
    interface = udev_device_get_sysattr_value(dev_int,"interface");

    dev = udev_device_get_parent_with_subsystem_devtype(
  	  dev, "usb", "usb_device");
    if (!dev) return 1;
    vid = udev_device_get_sysattr_value(dev, "idVendor");
    pid = udev_device_get_sysattr_value(dev, "idProduct");
    vendor = udev_device_get_sysattr_value(dev,"manufacturer");
    device = udev_device_get_sysattr_value(dev,"product");
    serial = udev_device_get_sysattr_value(dev,"serial");

    printf("VID/PID:%s/%s\n", vid, pid);
    if (!strcmp(vid, VID) && 
	!strcmp(pid, PID)) {
      fd = open(node, O_RDWR);
      if (fd < 0) return 1;
    }
    udev_device_unref(dev);
  }
  udev_enumerate_unref(enumerate);
  udev_unref(udev);
  if (fd < 0) return 1;

  n = 8;  // number of bytes to send per write
  while (1) {
    buf[0] = 0; // if first byte is zero it won't be transmitted
    for (i=0; i<n; i++) buf[i+1] = '0'+i;

    ret = write(fd, buf, n+1);
    if (ret != n+1)  { perror("write"); break; }

    ret = read(fd, buf, sizeof(buf));
    if (ret < 0)  { perror("read"); break; }
    printf("%.*s\n", ret, buf);
  }
}

